#include "CommandQueue.h"
#include "log.h"
#include <string.h>
#include <stdio.h>


CommandQueue::CommandQueue() {
	currentIdx = 0;
	nextCommandIdx = 0;
	inData = false;
}

void CommandQueue::load( char* filename ) {
	FILE* fp = fopen( filename, "r" );
	if ( fp != NULL ) {
		char line[1000];
		while ( fgets( &line[0], 1000, fp ) != NULL ) {
			if ( strlen( line ) > 0 ) {
				line[ strlen( line ) - 1 ] = 0;
			}
			queue( line );
		}
		fclose( fp );
	}
}

void CommandQueue::queue( char* command ) {
	if ( inData ) {
		if ( strncmp( command, "enddata", 7 ) == 0 ) {
			addCommand( EndData, command );
			inData = 0;
		} else {		
			addCommand( Data, command );
		}
	} else if ( inTemplate ) {
		if ( strncmp( command, "endtemplate", 11 ) == 0 ) {
			//log_message( "Queueing endtemplate" );
			addCommand( EndTemplate, command );
			inTemplate = 0;
		} else {
			//log_message( "Queueing template data" );
			addCommand( TemplateData, command );
		}
	} else if ( strncmp( command, "load ", 5 ) == 0 ) {
		addCommand( Load, command + 5 );
	} else if ( strncmp( command, "reset", 5 ) == 0 ) {
		addCommand( ReSetEngine, NULL );
	} else if ( strncmp( command, "show", 4 ) == 0 ) {
		addCommand( Show, NULL );
	} else if ( strncmp( command, "hide", 4 ) == 0 ) {
		addCommand( Hide, NULL );
	} else if ( strncmp( command, "left", 4 ) == 0 ) {
		addCommand( Left, NULL );
	} else if ( strncmp( command, "right", 5 ) == 0 ) {
		addCommand( Right, NULL );
	} else if ( strncmp( command, "inline ", 7 ) == 0 ) {
		addCommand( Inline, command + 7 );
	} else if ( strncmp( command, "in", 2 ) == 0 ) {
		addCommand( In, NULL );
	} else if ( strncmp( command, "out", 3 ) == 0 ) {
		addCommand( Out, NULL );
	} else if ( strncmp( command, "up", 2 ) == 0 ) {
		addCommand( Up, NULL );
	} else if ( strncmp( command, "down", 4 ) == 0 ) {
		addCommand( Down, NULL );
	} else if ( strncmp( command, "taller", 6 ) == 0 ) {
		addCommand( Taller, NULL );
	} else if ( strncmp( command, "shorter", 7 ) == 0 ) {
		addCommand( Shorter, NULL );
	} else if ( strncmp( command, "bigger", 6 ) == 0 ) {
		addCommand( Bigger, NULL );
	} else if ( strncmp( command, "smaller", 7 ) == 0 ) {
		addCommand( Smaller, NULL );
	} else if ( strncmp( command, "fatter", 6 ) == 0 ) {
		addCommand( Fatter, NULL );
	} else if ( strncmp( command, "thinner", 7 ) == 0 ) {
		addCommand( Thinner, NULL );
	} else if ( strncmp( command, "fadeout", 7 ) == 0 ) {
		addCommand( FadeOut, NULL );
	} else if ( strncmp( command, "fadein", 6 ) == 0 ) {
		addCommand( FadeIn, NULL );
	} else if ( strncmp( command, "cycle", 5 ) == 0 ) {
		addCommand( Cycle, NULL );
	} else if ( strncmp( command, "debug", 5 ) == 0 ) {
		addCommand( Debug, NULL );
	} else if ( strncmp( command, "setmax ", 7 ) == 0 ) {
		addCommand( SetMax, command + 7 );
	} else if ( strncmp( command, "setvalue ", 9 ) == 0 ) {
		addCommand( SetValue, command + 9 );
	} else if ( strncmp( command, "label ", 6 ) == 0 ) {
		addCommand( Label, command + 6 );
	} else if ( strncmp( command, "infodimension ", 14 ) == 0 ) {
		addCommand( InfoDimension, command + 14 );
	} else if ( strncmp( command, "startdata ", 10 ) == 0 ) {
		inData = 1;
		addCommand( StartData, command + 10 );
	} else if (( strncmp( command, "scale ", 6 ) == 0 ) && !inData ) {
		addCommand( Scale, command + 6 );
	} else if (strncmp( command, "setobject ", 10 ) == 0 ) {
		addCommand( SetObject, command + 10 );
	} else if ( strncmp( command, "starttemplate ", 14 ) == 0 ) {
		//log_message( "Queueing starttemplate" );
		inTemplate = 1;
		addCommand( StartTemplate, command + 14 );
	}
}

static char* process( char* s ) {
	int count = 0;
	while ( *s ) {
		if ((( *s == ':' ) || ( *s == ' ')) && ( count != 1 )) {
			*s = 0;
			s++;
			if ( *s == '_' ) {
				return( s + 1 );
			} else {
				return( s );
			}
		}
		s++;
		count++;
	}
	return( NULL );
}

void CommandQueue::addCommand( int cmd, char* str ) {
	if ( spaceAvailable() ) {
		command[ currentIdx ] = cmd;
		if ( str == NULL ) {
			param1[ currentIdx ] = NULL;
			param2[ currentIdx ] = NULL;
		} else {
			param1[ currentIdx ] = newstrcpy( str );

			// Label's get special handling, everything in label is param1
			if ( !inData && !inTemplate && ( cmd != Label )) {
				param2[ currentIdx ] = process( param1[ currentIdx ] );
			}
		}
		currentIdx = incValue( currentIdx );
	}
}

bool CommandQueue::hasCommand() {
	if ( currentIdx != nextCommandIdx ) {
		return( true );
	} else {
		return( false );
	}
}

char* CommandQueue::toStr( int command ) {
	switch ( command ) {
	case Load:
		return( "Load" );
	case ReSetEngine:
		return( "ReSetEngine");
	case Show:
		return( "Show" );
	case Hide:
		return( "Hide" );
	case InfoDimension:
		return( "InfoDimension" );
	case StartData:
		return( "StartData" );
	case EndData:
		return( "EndData" );
	case Data:
		return( "Data" );
	case Scale:
		return( "Scale" );
	case SetObject:
		return( "SetObject" );
	case Left:
		return( "Left" );
	case Right:
		return( "Right" );
	case Up:
		return( "Up" );
	case Down:
		return( "Down" );
	case In:
		return( "In" );
	case Out:
		return( "Out" );
	case Taller:
		return( "Taller" );
	case Shorter:
		return( "Bigger" );
	case Smaller:
		return( "Smaller" );
	case SetMax:
		return( "SetMax" );
	case SetValue:
		return( "SetValue" );
	case Cycle:
		return( "Cycle" );
	case Fatter:
		return( "Fatter" );
	case Thinner:
		return( "Thinner" );
	case FadeOut:
		return( "FadeOut" );
	case FadeIn:
		return( "FadeIn" );
	case Inline:
		return( "Inline" );
	case Debug:
		return( "Debug" );
	default:
		return( "unknown" );
	}
}


Command* CommandQueue::getNextCommand() {
	if ( inData ) {
		//log_message( "no command" );
		return( NULL );
	}
	if ( hasCommand() ) {
		if ( debug ) log_message1( "we have a command", toStr(command[ nextCommandIdx ]) );
		Command* result = new Command( command[ nextCommandIdx ], param1[ nextCommandIdx ], param2[ nextCommandIdx ] );
		nextCommandIdx = incValue( nextCommandIdx );
		return( result );
	} else {
		//log_message( "no command #2" );
		return( NULL );
	}

}
Command* CommandQueue::unconditionalGetNextCommand() {
	if ( hasCommand() ) {
		//log_message( "we have a command" );
		Command* result = new Command( command[ nextCommandIdx ], param1[ nextCommandIdx ], param2[ nextCommandIdx ] );
		nextCommandIdx = incValue( nextCommandIdx );
		return( result );
	} else {
		//log_message( "no command #2" );
		return( NULL );
	}

}
Command::Command( int c, char* p1, char* p2 ) {
	command = c;
	param1 = p1;
	param2 = p2;
}

#include <malloc.h>
Command::~Command() {
	/*if ( param1 != NULL ) {
		free( param1 );
	}*/
	/*if ( param2 != NULL ) {
		free( param2 );
	}*/
}

int CommandQueue::incValue( int n ) {
	n++;
	if ( n == COMMAND_MAX ) {
		return( 0 );
	}
	return( n );
}

bool CommandQueue::spaceAvailable() {
	if ( incValue( currentIdx ) != nextCommandIdx ) {
		return( true );
	} else {
		return( false );
	}
}
